${ws}{${te_lnk.te_classGeneratedName} * selected = ${cast}${te_lnk.left}->${te_lnk.linkage};
${ws}if ( ( 0 != selected ) && ( ${te_select_related.where_clause} ) ) {
${ws}  ${te_select_related.result_var} = selected;
${ws}}}
